/*
 * Copyright (C) 2010 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

//BEGIN_INCLUDE(all)
#include <stdlib.h>
#include <android/log.h>
#include <android_native_app_glue.h>
#include "ewrte.h"
#include "ewmain.h"
#include "gfx_system_android.h"

/* multi-touch events provided by Android */
static int TouchEventXPos[ NO_OF_FINGERS_ANDROID ];
static int TouchEventYPos[ NO_OF_FINGERS_ANDROID ];
static int TouchEventDown[ NO_OF_FINGERS_ANDROID ];

static ANativeWindow* NativeWindow = 0;

/**
 * Our saved state data.
 */
struct saved_state
{
  int  dummy;
};

/**
 * Shared state for our app.
 */
struct engine
{
  struct android_app *     app;
  struct saved_state       state;
};


/**
 * Process the next input event.
 */
static int32_t HandleInput ( struct android_app* app, AInputEvent* event )
{
  if ( AInputEvent_getType( event ) == AINPUT_EVENT_TYPE_MOTION &&
       AInputEvent_getSource( event ) == AINPUT_SOURCE_TOUCHSCREEN )
  {
    int i;
    int id;
    int action = AKeyEvent_getAction( event ) & AMOTION_EVENT_ACTION_MASK;
    int count = AMotionEvent_getPointerCount( event );

    /* Android reports the status for all currently active fingers
       => set first all fingers to status 'up' */
    for ( i = 0; i < NO_OF_FINGERS_ANDROID; i++ )
      TouchEventDown[ i ] = 0;

    /* read touch event data */
    for ( i = 0; i < count; i++ )
    {
      id = AMotionEvent_getPointerId( event, i );
      if (( id < 0 ) || ( id >= NO_OF_FINGERS_ANDROID ))
        continue;

      TouchEventXPos[ id ] = AMotionEvent_getX( event, i );
      TouchEventYPos[ id ] = AMotionEvent_getY( event, i );
      if (( action == AMOTION_EVENT_ACTION_DOWN ) ||
          ( action == AMOTION_EVENT_ACTION_POINTER_DOWN ) ||
          ( action == AMOTION_EVENT_ACTION_MOVE ))
        TouchEventDown[ id ] = 1;
    }
    return 1;
  }
  return 0;
}


/**
 * Process the next main command.
 */
static void HandleCmd ( struct android_app* app, int32_t cmd )
{
    struct engine * engine = (struct engine*)app->userData;

    switch ( cmd )
    {
      case APP_CMD_SAVE_STATE:
        // The system has asked us to save our current state.  Do so.
        engine->app->savedState = malloc( sizeof( struct saved_state ) );
        *((struct saved_state*)engine->app->savedState) = engine->state;
        engine->app->savedStateSize = sizeof( struct saved_state );
        EwPrint( "Application stated saved\n" );
        break;

      case APP_CMD_INIT_WINDOW:
        // The window is being shown, get it ready.
        if ( engine->app->window != NULL )
        {
          NativeWindow = engine->app->window;
          EwInit();
          EwPrintSystemInfo();
          EwPrint( "Application window initialized\n" );
        }
        break;

      case APP_CMD_TERM_WINDOW:
        // The window is being hidden or closed, clean it up.
        EwDone();
        EwPrint( "Application window terminated\n" );
        break;

      default:
        break;
  }
}


/**
 * This is the main entry point of a native application that is using
 * android_native_app_glue.  It runs in its own thread, with its own
 * event loop for receiving input events and doing other things.
 */
void android_main( struct android_app* app )
{
  int i;
  struct engine  engine;
  unsigned int stack;

  /* determine the stack area and provide it to the Runtime Environmnet */
  EwSetStackAddressArea( &stack, (void*)((intptr_t)&stack - EW_GUI_THREAD_STACK_SIZE ));

  memset( &engine, 0x00, sizeof( engine ) );
  app->userData     = &engine;
  app->onAppCmd     = HandleCmd;
  app->onInputEvent = HandleInput;
  engine.app        = app;

  /* clear touch event data */
  for ( i = 0; i < NO_OF_FINGERS_ANDROID; i++ )
  {
    TouchEventXPos[ i ] = 0;
    TouchEventYPos[ i ] = 0;
    TouchEventDown[ i ] = 0;
  }

  EwPrint( "\nApplication started\n" );

  if ( app->savedState != NULL )
  {
    // We are starting with a previous saved state; restore from it.
    engine.state = *(struct saved_state*)app->savedState;
  }

  // loop waiting for stuff to do.
  while ( 1 )
  {
    // Read all pending events.
    int                          ident;
    int                          events;
    struct android_poll_source * source;


    // we loop until all events are read, then continue
    // to draw the next frame of animation.
    while ( ( ident = ALooper_pollAll( 0, NULL, &events, (void**)&source)) >= 0 )
    {
      // Process this event.
      if ( source != NULL )
      {
        source->process( app, source );
      }
    }

    EwProcess();
    // Check if we are exiting.
    if ( app->destroyRequested != 0 )
      return;
  }
}


/*******************************************************************************
* FUNCTION:
*   AndroidGetNaitveWindow
*
* DESCRIPTION:
*   The function AndroidGetNaitveDisplay returns the native Android window used
*   for EGL initialization.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   Returns the native window.
*
*******************************************************************************/
void* AndroidGetNaitveWindow( void )
{
  return NativeWindow;
}


/*******************************************************************************
* FUNCTION:
*   AndroidGetTouchState
*
* DESCRIPTION:
*   The function AndroidGetTouchState returns the current touch state and the
*   touch position.
*
* ARGUMENTS:
*   aFinger  - Number of finger to return the data.
*   aTouchX  - Pointer to return touch position in X direction.
*   aTouchY  - Pointer to return touch position in Y direction.
*
* RETURN VALUE:
*   Returns 1 if currently touched, 0 otherwise.
*
*******************************************************************************/
int AndroidGetTouchState( int aFinger, int* aTouchX, int* aTouchY )
{
  /* check for valid finger number */
  if (( aFinger < 0 ) || ( aFinger > NO_OF_FINGERS_ANDROID ))
    return 0;

  if ( aTouchX )
    *aTouchX = TouchEventXPos[ aFinger ];
  if ( aTouchY )
    *aTouchY = TouchEventYPos[ aFinger ];

  return ( TouchEventDown[ aFinger ] );
}

//END_INCLUDE(all)
