/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the display hardware of the board
*   and to provide the necessary access to update the display content.
*   The color format of the framebuffer has to correspond to the color format
*   of the Graphics Engine.
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation according your particular hardware.
*
*******************************************************************************/

#include <GLES2/gl2.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>

#include <android_native_app_glue.h>

#include "ewrte.h"
#include "ewgfx.h"

#include "ew_bsp_display.h"
#include "gfx_system_android.h"

typedef struct
{
  EGLDisplay  EglDisplay;
  EGLSurface  EglSurface;
  EGLContext  EglContext;
} XDisplayContext;

static XDisplayContext DisplayContext;


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayInit
*
* DESCRIPTION:
*   The function EwBspDisplayInit initializes the display hardware and returns
*   the display parameter.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  ANativeWindow *       nativeWindow;
  int                   guiWidth;
  int                   guiHeight;
  int                   viewportX;
  int                   viewportY;
  int                   viewportWidth;
  int                   viewportHeight;
  const char *          eglVendor;
  EGLNativeDisplayType  nativeDisplayType   = EGL_DEFAULT_DISPLAY;
  EGLNativeWindowType   eglNativeWindowType = 0;
  EGLDisplay            eglDisplay          = 0;
  EGLConfig             eglConfig           = 0;
  EGLSurface            eglSurface          = 0;
  EGLContext            eglContext          = 0;
  EGLint                num_configs;
  GLint                 framebuffer;
  EGLint                eglFormat;
  EGLint                width;
  EGLint                height;
  EGLint                contextAttribs[]  =
  {
    EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE
  };
  EGLint                configAttribs[] =
  {
    EGL_SURFACE_TYPE,      EGL_WINDOW_BIT,
    EGL_RENDERABLE_TYPE,   EGL_OPENGL_ES2_BIT,
    EGL_BUFFER_SIZE,       32,
    EGL_COLOR_BUFFER_TYPE, EGL_RGB_BUFFER,
    EGL_RED_SIZE,          8,
    EGL_GREEN_SIZE,        8,
    EGL_BLUE_SIZE,         8,
    EGL_ALPHA_SIZE,        8,
    EGL_NONE
  };


  /* check and clean display info structure */
  if ( !aDisplayInfo )
    return 0;
  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));


  nativeWindow = ( ANativeWindow * ) AndroidGetNaitveWindow();
  if ( ! nativeWindow )
  {
    EwPrint( "ERROR - No native window\n" );
    return 0;
  }

  eglDisplay = eglGetDisplay( nativeDisplayType );
  eglInitialize( eglDisplay, 0, 0 );
  eglBindAPI( EGL_OPENGL_ES_API );
  eglChooseConfig( eglDisplay, configAttribs, &eglConfig, 1, &num_configs );

  if ( num_configs != 1 )
  {
    EwPrint( "ERROR - Wrong number of EGL configs: %d\n", num_configs );
    return 0;
  }

  /* EGL_NATIVE_VISUAL_ID is an attribute of the EGLConfig that is
   * guaranteed to be accepted by ANativeWindow_setBuffersGeometry().
   * As soon as we picked a EGLConfig, we can safely reconfigure the
   * ANativeWindow buffers to match, using EGL_NATIVE_VISUAL_ID. */
  eglGetConfigAttrib( eglDisplay, eglConfig, EGL_NATIVE_VISUAL_ID, &eglFormat );
  ANativeWindow_setBuffersGeometry( nativeWindow, 0, 0, eglFormat );

  eglSurface = eglCreateWindowSurface( eglDisplay, eglConfig, nativeWindow, 0 );
  eglContext = eglCreateContext( eglDisplay, eglConfig, EGL_NO_CONTEXT, contextAttribs );
  #if EW_PERFORM_FULLSCREEN_UPDATE == 0
    EGLBoolean  eglResult;
    eglResult = eglSurfaceAttrib( eglDisplay, eglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_PRESERVED );
    if ( eglResult != EGL_TRUE )
      EwPrint( "\nWARNING: egl swap behaviour - buffer preservation not supported (egl error: 0x%04X)\n" \
        "Use macro EW_PERFORM_FULLSCREEN_UPDATE = 1\n\n", eglGetError());
  #else
    eglSurfaceAttrib( eglDisplay, eglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_DESTROYED );
  #endif

  eglMakeCurrent( eglDisplay, eglSurface, eglSurface, eglContext );

  /* set swap interval to display framerate */
  eglSwapInterval( eglDisplay, 1 );

  /* store EGL display and surface in display context */
  DisplayContext.EglDisplay = eglDisplay;
  DisplayContext.EglSurface = eglSurface;
  DisplayContext.EglContext = eglContext;

  /* clear entire screen with black color */
  glClearColor( 0.0, 0.0, 0.0, 1.0 );
  glClear( GL_COLOR_BUFFER_BIT );
  glFlush();
  eglSwapBuffers( eglDisplay, eglSurface );

  /* get the framebuffer and its size in pixel */
  glGetIntegerv( GL_FRAMEBUFFER_BINDING, &framebuffer);
  eglQuerySurface( eglDisplay, eglSurface, EGL_WIDTH, &width );
  eglQuerySurface( eglDisplay, eglSurface, EGL_HEIGHT, &height );

  /* adjust size of gui if viewport is rotated */
  #if (( EW_VIEWPORT_ROTATION == 90 ) || ( EW_VIEWPORT_ROTATION == 270 ))
    guiWidth  = aGuiHeight;
    guiHeight = aGuiWidth;
  #else
    guiWidth  = aGuiWidth;
    guiHeight = aGuiHeight;
  #endif

  #if EW_VIEWPORT_AUTOSCALE == 1

    if (((float)width / (float)guiWidth ) < ((float)height / (float)guiHeight ))
    {
      /* modify height of viewport to keep the aspect ratio */
      viewportWidth  = width;
      viewportHeight = guiHeight * width / guiWidth;
      viewportX      = 0;
      viewportY      = ( height - viewportHeight ) / 2;
    }
    else
    {
      /* modify width of viewport to keep the aspect ratio */
      viewportWidth  = guiWidth * height / guiHeight;
      viewportHeight = height;
      viewportX      = ( width - viewportWidth ) / 2;
      viewportY      = 0;
    }

  #else

    viewportWidth  = guiWidth;
    viewportHeight = guiHeight;
    viewportX      = ( width  - guiWidth ) / 2;
    viewportY      = ( height - guiHeight ) / 2;

  #endif

  /* return the current display configuration */
  aDisplayInfo->FrameBuffer    = (void*)(intptr_t)framebuffer;
  aDisplayInfo->DisplayWidth   = width;
  aDisplayInfo->DisplayHeight  = height;
  aDisplayInfo->ViewportX      = viewportX;
  aDisplayInfo->ViewportY      = viewportY;
  aDisplayInfo->ViewportWidth  = viewportWidth;
  aDisplayInfo->ViewportHeight = viewportHeight;
  aDisplayInfo->Context        = &DisplayContext;

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayDone
*
* DESCRIPTION:
*   The function EwBspDisplayDone deinitializes the display hardware.
*
* ARGUMENTS:
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  XDisplayContext* context;

  if ( !aDisplayInfo )
    return;

  context = (XDisplayContext*) aDisplayInfo->Context;

  if ( !context )
    return;

  /* shutdown the EGL sub-system */
  eglMakeCurrent( context->EglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT ) ;
  eglDestroySurface( context->EglDisplay, context->EglSurface );
  eglDestroyContext(context->EglDisplay, context->EglContext);
  eglTerminate( context->EglDisplay );
  eglReleaseThread();
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplaySwapBuffers
*
* DESCRIPTION:
*   The function EwBspDisplaySwapBuffers is called from the completion callback
*   of the viewport. The function ensures that the screen content is shown by
*   swapping the EGL buffers.
*
* ARGUMENTS:
*   aContext - Display context including EGL display and EGL surface.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplaySwapBuffers( void* aContext )
{
  XDisplayContext* context = (XDisplayContext*)aContext;

  if ( !context )
    return;

  /* perform the swap if there was something drawn on the screen */
  eglSwapBuffers( context->EglDisplay, context->EglSurface );
}


/* mli, msy */
