/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the touch driver of the display
*   hardware and to receive the touch events for the UI application.
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation according your particular hardware.
*
*******************************************************************************/

#include <string.h>

#include "ewconfig.h"
#include "ewrte.h"

#include "ew_bsp_display.h"
#include "ew_bsp_touch.h"

#include "gfx_system_android.h"

#define NO_OF_FINGERS                   NO_OF_FINGERS_ANDROID
#define DELTA_TOUCH                     16
#define DELTA_TIME                      500

/* additional touch flag to indicate idle state */
#define EW_BSP_TOUCH_IDLE               0

/* additional touch flag to indicate hold state */
#define EW_BSP_TOUCH_HOLD               4

/* structure to store internal touch information for one finger */
typedef struct
{
  int           XPos;      /* horizontal position in pixel */
  int           YPos;      /* vertical position in pixel */
  unsigned long Ticks;     /* time of recent touch event */
  unsigned char TouchId;   /* constant touch ID provided by touch controller */
  unsigned char State;     /* current state within a touch cycle */
} XTouchData;

static int           GuiWidth         =  0; /* width of the GUI application (EwScreenSize.X) */
static int           GuiHeight        =  0; /* height of the GUI application (EwScreenSize.Y) */
static int           ViewportX        =  0; /* x origin of the viewport (= area where the GUI will be drawn) */
static int           ViewportY        =  0; /* y origin of the viewport (= area where the GUI will be drawn) */
static int           ViewportWidth    =  0; /* width of the viewport */
static int           ViewportHeight   =  0; /* height of the viewport */
static int           DisplayWidth     =  0; /* width of the display */
static int           DisplayHeight    =  0; /* height of the display */
static int           IsInitalized     =  0;

static XTouchEvent   TouchEvent[ NO_OF_FINGERS ];
static XTouchData    TouchData[ NO_OF_FINGERS ];
static int           TouchId[ NO_OF_FINGERS ];


/*******************************************************************************
* FUNCTION:
*   EwBspTouchInit
*
* DESCRIPTION:
*   Initalizes the touch driver interface. The provided display information is
*   used to configure the touch driver interface so that a proper mapping of
*   touch coordinates to GUI coordinates can be done.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspTouchInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  /* clear all touch state variables */
  memset( TouchData, 0, sizeof( TouchData ));
  memset( TouchEvent, 0, sizeof( TouchEvent ));
  memset( TouchId, -1, sizeof( TouchId ));

  /* adjust size of gui if viewport is rotated */
  #if (( EW_VIEWPORT_ROTATION == 90 ) || ( EW_VIEWPORT_ROTATION == 270 ))

    GuiWidth  = aGuiHeight;
    GuiHeight = aGuiWidth;

  #else

    GuiWidth  = aGuiWidth;
    GuiHeight = aGuiHeight;

  #endif

  /* check display info structure */
  if ( !aDisplayInfo )
  {
    EwPrint( "EwBspTouchInit: Invalid DisplayInfo!\n" );
    return 0;
  }

  /* take physical size of display and location of viewport from provided display info structure */
  ViewportX      = aDisplayInfo->ViewportX;
  ViewportY      = aDisplayInfo->ViewportY;
  ViewportWidth  = aDisplayInfo->ViewportWidth;
  ViewportHeight = aDisplayInfo->ViewportHeight;
  DisplayWidth   = aDisplayInfo->DisplayWidth;
  DisplayHeight  = aDisplayInfo->DisplayHeight;

  if ( ViewportWidth == 0 || ViewportHeight == 0 )
  {
    EwPrint( "EwBspTouchInit: Invalid Viewport size!\n" );
    return 0;
  }

  IsInitalized = 1;
  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspTouchDone
*
* DESCRIPTION:
*   Terminates the touch driver.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspTouchDone( void )
{
  IsInitalized = 0;
}


/*******************************************************************************
* FUNCTION:
*   EwBspTouchGetEvents
*
* DESCRIPTION:
*   The function EwBspTouchGetEvents reads the current touch positions from the
*   touch driver and returns the current touch position and touch status of the
*   different fingers. The returned number of touch events indicates the number
*   of XTouchEvent that contain position and status information.
*   The orientation of the touch positions is adjusted to match GUI coordinates.
*   If the hardware supports only single touch, the finger number is always 0.
*
* ARGUMENTS:
*   aTouchEvent - Pointer to return array of XTouchEvent.
*
* RETURN VALUE:
*   Returns the number of detected touch events, otherwise 0.
*
*******************************************************************************/
int EwBspTouchGetEvents( XTouchEvent** aTouchEvent )
{
  int           touchDown;
  int           touchX;
  int           touchY;
  int           x, y;
  int           t;
  int           f;
  unsigned long ticks;
  int           noOfEvents = 0;
  int           finger;
  char          identified[ NO_OF_FINGERS ];
  XTouchData*   touch;


  if ( !IsInitalized )
    return 0;

  /* all fingers have the state unidentified */
  memset( identified, 0, sizeof( identified ));

  /* get current time in ms */
  ticks = EwGetTicks();

  /* iterate through all potential touch events */
  for ( t = 0; t < NO_OF_FINGERS; t++ )
  {
    touchDown = AndroidGetTouchState( t, &touchX, &touchY );
    /* check for valid coordinates */
    if ( touchDown )
      TouchId[ t ] = t;

    /* check for valid touch id */
    if ( TouchId[ t ] < 0 )
      continue;

    #ifdef EW_PRINT_TOUCH_DATA

      EwPrint( "Raw touch data: id %d, x %d, y %d, down %d\n", TouchId[ t ], touchX, touchY, touchDown );

    #endif

    /* convert display coordinates into GUI coordinates */
    #if ( EW_VIEWPORT_ROTATION == 90 )

      x = ( touchY - ViewportY ) * GuiHeight / ViewportHeight;
      y = ( ViewportWidth - ( touchX - ViewportX )) * GuiWidth / ViewportWidth;

    #elif ( EW_VIEWPORT_ROTATION == 270 )

      x = ( ViewportHeight - ( touchY - ViewportY )) * GuiHeight / ViewportHeight;
      y = ( touchX - ViewportX ) * GuiWidth / ViewportWidth;

    #elif ( EW_VIEWPORT_ROTATION == 180 )

      x = ( ViewportWidth - ( touchX - ViewportX )) * GuiWidth / ViewportWidth;
      y = ( ViewportHeight - ( touchY - ViewportY )) * GuiHeight / ViewportHeight;

    #else

      x = ( touchX - ViewportX ) * GuiWidth / ViewportWidth;
      y = ( touchY - ViewportY ) * GuiHeight / ViewportHeight;

    #endif

    #ifdef EW_PRINT_TOUCH_DATA

      EwPrint( "GUI coordinates: x %d, y %d\n", x, y );

    #endif

    /* iterate through all fingers to find a finger that matches with the provided touch event */
    for ( finger = -1, f = 0; f < NO_OF_FINGERS; f++ )
    {
      touch = &TouchData[ f ];

      /* check if the finger is already active */
      if (( touch->State != EW_BSP_TOUCH_IDLE ) && ( touch->TouchId == TouchId[ t ] ))
      {
        finger = f;
        break;
      }

      if ( touchDown == 0 )
        continue;

      /* check if the finger was used within the recent time span and if the touch position is in the vicinity */
      if (( touch->State == EW_BSP_TOUCH_IDLE ) && ( ticks < touch->Ticks + DELTA_TIME )
        && ( x > touch->XPos - DELTA_TOUCH ) && ( x < touch->XPos + DELTA_TOUCH )
        && ( y > touch->YPos - DELTA_TOUCH ) && ( y < touch->YPos + DELTA_TOUCH ))
        finger = f;

      /* otherwise take the first free finger */
      if (( touch->State == EW_BSP_TOUCH_IDLE ) && ( finger == -1 ))
        finger = f;
    }

    /* determine the state within a touch cycle and assign the touch parameter to the found finger */
    if ( finger >= 0 )
    {
      touch = &TouchData[ finger ];
      identified[ finger ] = 1;

      if ( touchDown == 1 )
      {
        /* check for start of touch cycle */
        if ( touch->State == EW_BSP_TOUCH_IDLE )
          touch->State = EW_BSP_TOUCH_DOWN;
        else
        {
          /* check if the finger has moved */
          if (( touch->XPos != x ) || ( touch->YPos != y ))
            touch->State = EW_BSP_TOUCH_MOVE;
          else
            touch->State = EW_BSP_TOUCH_HOLD;
        }
      }
      else if ( touch->State != EW_BSP_TOUCH_IDLE )
        touch->State = EW_BSP_TOUCH_UP;

      /* store current touch parameter */
      touch->XPos    = x;
      touch->YPos    = y;
      touch->TouchId = TouchId[ t ];
      touch->Ticks   = ticks;

      /* clear touch id when touch cycle is completed - no further data required for this id */
      if ( touch->State == EW_BSP_TOUCH_UP )
        TouchId[ t ] = -1;
    }
  }

  /* prepare sequence of touch events suitable for Embedded Wizard GUI application */
  for ( f = 0; f < NO_OF_FINGERS; f++ )
  {
    touch = &TouchData[ f ];

    /* begin of a touch cycle */
    if ( identified[ f ] && ( touch->State == EW_BSP_TOUCH_DOWN ))
      TouchEvent[ noOfEvents ].State = EW_BSP_TOUCH_DOWN;

    /* move within a touch cycle */
    else if ( identified[ f ] && ( touch->State == EW_BSP_TOUCH_MOVE ))
      TouchEvent[ noOfEvents ].State = EW_BSP_TOUCH_MOVE;

    /* end of a touch cycle */
    else if (( !identified[ f ] && ( touch->State != EW_BSP_TOUCH_IDLE )) || ( touch->State == EW_BSP_TOUCH_UP ))
    {
      TouchEvent[ noOfEvents ].State = EW_BSP_TOUCH_UP;
      touch->State = EW_BSP_TOUCH_IDLE;
    }
    else
      continue;

    TouchEvent[ noOfEvents ].XPos   = touch->XPos;
    TouchEvent[ noOfEvents ].YPos   = touch->YPos;
    TouchEvent[ noOfEvents ].Finger = f;

    #ifdef EW_PRINT_TOUCH_EVENTS

      EwPrint( "Touch event for finger %d with state %d ( %4d, %4d )\n", f, TouchEvent[ noOfEvents ].State, TouchEvent[ noOfEvents ].XPos, TouchEvent[ noOfEvents ].YPos );

    #endif

    noOfEvents++;
  }

  /* return the prepared touch events and the number of prepared touch events */
  if ( aTouchEvent )
    *aTouchEvent = TouchEvent;

  return noOfEvents;
}


/* msy */
